@@ -8,7 +8,8 @@ from pandaclient import PrunScript
 from pandaclient import PhpoScript
 
 from idds.workflowv2.workflow import Workflow, Condition, AndCondition, OrCondition
-from idds.atlas.workflow.atlaspandawork import ATLASPandaWork
+from idds.atlas.workflowv2.atlaspandawork import ATLASPandaWork
+from idds.atlas.workflowv2.atlaslocalpandawork import ATLASLocalPandaWork
 
 
 # extract argument value from execution string
@@ -133,7 +134,7 @@ class Node (object):
             for k in ['opt_exec', 'opt_args']:
                 test_str = dict_inputs.get(k)
                 if test_str:
-                    m = re.search(r'%%DS\d+%%', test_str)
+                    m = re.search(r'%{DS(\d+|\*)}', test_str)
                     if m:
                         return False, '{} is unresolved in {}'.format(m.group(0), k)
             if self.type == 'prun':
@@ -141,10 +142,21 @@ class Node (object):
                     if k not in ['opt_inDS', 'opt_inDsType', 'opt_secondaryDSs', 'opt_secondaryDsTypes',
                                  'opt_args', 'opt_exec', 'opt_useAthenaPackages', 'opt_containerImage']:
                         return False, 'unknown input parameter {}'.format(k)
+            elif self.type in ['junction', 'reana']:
+                for k in dict_inputs:
+                    if k not in ['opt_inDS', 'opt_inDsType', 'opt_args', 'opt_exec', 'opt_containerImage']:
+                        return False, 'unknown input parameter {}'.format(k)
             elif self.type == 'phpo':
                 for k in dict_inputs:
                     if k not in ['opt_trainingDS', 'opt_trainingDsType', 'opt_args']:
                         return False, 'unknown input parameter {}'.format(k)
+        elif self.type == 'workflow':
+            reserved_params = ['i']
+            loop_global, workflow_global = self.get_global_parameters()
+            if loop_global:
+                for k in reserved_params:
+                    if k in loop_global:
+                        return False, 'parameter {} cannot be used since it is reserved by the system'.format(k)
         return True, ''
 
     # string representation
@@ -168,7 +180,7 @@ class Node (object):
         return "ID:{} Name:{} Type:{}".format(self.id, self.name, self.type)
 
     # resolve workload-specific parameters
-    def resolve_params(self, task_template=None, id_map=None):
+    def resolve_params(self, task_template=None, id_map=None, workflow=None):
         if self.type == 'prun':
             dict_inputs = self.convert_dict_inputs()
             if 'opt_secondaryDSs' in dict_inputs:
@@ -184,8 +196,11 @@ class Node (object):
                                 break
                 # resolve secondary dataset names
                 for ds_name, ds_type in zip(dict_inputs['opt_secondaryDSs'], dict_inputs['opt_secondaryDsTypes']):
-                    src = "%%DS{}%%".format(idx)
-                    dst = "{}.{}".format(ds_name, ds_type)
+                    if '*' in ds_type:
+                        ds_type = ds_type.replace('*', 'XYZ')
+                        ds_type += '.tgz'
+                    src = "%{{DS{}}}".format(idx)
+                    dst = "{}_{}/".format(ds_name, ds_type)
                     dict_inputs['opt_exec'] = re.sub(src, dst, dict_inputs['opt_exec'])
                     dict_inputs['opt_args'] = re.sub(src, dst, dict_inputs['opt_args'])
                     idx += 1
@@ -195,20 +210,21 @@ class Node (object):
                     elif k.endswith('opt_args'):
                         v['value'] = dict_inputs['opt_args']
         if self.is_leaf and task_template:
-            self.task_params = self.make_task_params(task_template, id_map)
-        [n.resolve_params(task_template, id_map) for n in self.sub_nodes]
+            self.task_params = self.make_task_params(task_template, id_map, workflow)
+        [n.resolve_params(task_template, id_map, self) for n in self.sub_nodes]
 
     # create task params
-    def make_task_params(self, task_template, id_map):
+    def make_task_params(self, task_template, id_map, workflow_node):
         # task name
         for k, v in six.iteritems(self.outputs):
             task_name = v['value']
             break
-        if self.type == 'prun':
+        if self.type in ['prun', 'junction', 'reana']:
             dict_inputs = self.convert_dict_inputs(skip_suppressed=True)
             # check type
             use_athena = False
-            if 'opt_useAthenaPackages' in dict_inputs and dict_inputs['opt_useAthenaPackages']:
+            if 'opt_useAthenaPackages' in dict_inputs and dict_inputs['opt_useAthenaPackages'] \
+                    and self.type != 'reana':
                 use_athena = True
             container_image = None
             if 'opt_containerImage' in dict_inputs and dict_inputs['opt_containerImage']:
@@ -219,8 +235,20 @@ class Node (object):
                 task_params = copy.deepcopy(task_template['container'])
             task_params['taskName'] = task_name
             # cli params
-            com = ['prun', '--exec', dict_inputs['opt_exec'], *shlex.split(dict_inputs['opt_args'])]
-            in_ds_str = None
+            com = ['prun']
+            if self.type == 'junction':
+                # add default output for junction
+                if 'opt_args' not in dict_inputs:
+                    dict_inputs['opt_args'] = ''
+                results_json = "results.json"
+                if '--outputs' not in dict_inputs['opt_args']:
+                    dict_inputs['opt_args'] += ' --outputs {}'.format(results_json)
+                else:
+                    m = re.search('(--outputs)( +|=)([^ ]+)', dict_inputs['opt_args'])
+                    if results_json not in m.group(3):
+                        tmp_dst = m.group(1) + '=' + m.group(3) + ',' + results_json
+                        dict_inputs['opt_args'] = re.sub(m.group(0), tmp_dst, dict_inputs['opt_args'])
+            com += shlex.split(dict_inputs['opt_args'])
             if 'opt_inDS' in dict_inputs and dict_inputs['opt_inDS']:
                 if isinstance(dict_inputs['opt_inDS'], list):
                     is_list_in_ds = True
@@ -244,13 +272,63 @@ class Node (object):
                                 break
                 else:
                     in_ds_suffix = dict_inputs['opt_inDsType']
+                    if '*' in in_ds_suffix:
+                        in_ds_suffix = in_ds_suffix.replace('*', 'XYZ') + '.tgz'
                 if is_list_in_ds:
-                    in_ds_str = ','.join(['{}_{}/'.format(s1, s2) if s2 else s1 for s1, s2
-                                          in zip(dict_inputs['opt_inDS'], in_ds_suffix)])
+                    list_in_ds = ['{}_{}/'.format(s1, s2) if s2 else s1 for s1, s2
+                                  in zip(dict_inputs['opt_inDS'], in_ds_suffix)]
                 else:
-                    in_ds_str = '{}_{}/'.format(dict_inputs['opt_inDS'], in_ds_suffix) if in_ds_suffix \
-                        else dict_inputs['opt_inDS']
-                com += ['--inDS', in_ds_str, '--notExpandInDS', '--notExpandSecDSs']
+                    list_in_ds = ['{}_{}/'.format(dict_inputs['opt_inDS'], in_ds_suffix) if in_ds_suffix \
+                                      else dict_inputs['opt_inDS']]
+                if self.type not in ['junction', 'reana']:
+                    in_ds_str = ','.join(list_in_ds)
+                    com += ['--inDS', in_ds_str, '--notExpandInDS', '--notExpandSecDSs']
+                else:
+                    # replace placeholders in opt_exec and opt_args
+                    for idx, dst in enumerate(list_in_ds):
+                        src = "%{{DS{}}}".format(idx)
+                        if 'opt_exec' in dict_inputs:
+                            dict_inputs['opt_exec'] = re.sub(src, dst, dict_inputs['opt_exec'])
+                        if 'opt_args' in dict_inputs:
+                            dict_inputs['opt_args'] = re.sub(src, dst, dict_inputs['opt_args'])
+                    if list_in_ds:
+                        src = r"%{DS\*}"
+                        if 'opt_exec' in dict_inputs:
+                            dict_inputs['opt_exec'] = re.sub(src, ','.join(list_in_ds), dict_inputs['opt_exec'])
+                        if 'opt_args' in dict_inputs:
+                            dict_inputs['opt_args'] = re.sub(src, ','.join(list_in_ds), dict_inputs['opt_args'])
+                    for k, v in six.iteritems(self.inputs):
+                        if k.endswith('opt_exec'):
+                            v['value'] = dict_inputs['opt_exec']
+                        elif k.endswith('opt_args'):
+                            v['value'] = dict_inputs['opt_args']
+            # global parameters
+            if workflow_node:
+                tmp_global, tmp_workflow_global = workflow_node.get_global_parameters()
+                src_dst_list = []
+                # looping globals
+                if tmp_global:
+                    for k in tmp_global:
+                        tmp_src = '%{{{}}}'.format(k)
+                        tmp_dst = '___idds___user_{}___'.format(k)
+                        src_dst_list.append((tmp_src, tmp_dst))
+                # workflow globls
+                if tmp_workflow_global:
+                    for k, v in six.iteritems(tmp_workflow_global):
+                        tmp_src = '%{{{}}}'.format(k)
+                        tmp_dst = '{}'.format(v)
+                        src_dst_list.append((tmp_src, tmp_dst))
+                # iteration count
+                tmp_src = '%{i}'
+                tmp_dst = '___idds___num_run___'
+                src_dst_list.append((tmp_src, tmp_dst))
+                # replace
+                for tmp_src, tmp_dst in src_dst_list:
+                    if 'opt_exec' in dict_inputs:
+                        dict_inputs['opt_exec'] = re.sub(tmp_src, tmp_dst, dict_inputs['opt_exec'])
+                    if 'opt_args' in dict_inputs:
+                        dict_inputs['opt_args'] = re.sub(tmp_src, tmp_dst, dict_inputs['opt_args'])
+            com += ['--exec', dict_inputs['opt_exec']]
             com += ['--outDS', task_name]
             if container_image:
                 com += ['--containerImage', container_image]
@@ -273,7 +351,8 @@ class Node (object):
             for p_key, p_value in six.iteritems(parsed_params):
                 if p_key in ['buildSpec']:
                     continue
-                if p_key not in task_params:
+                if p_key not in task_params or p_key in ['log', 'container_name', "multiStepExec",
+                                                         "site", "excludedSite", "includedSite"]:
                     task_params[p_key] = p_value
                 elif p_key == 'architecture':
                     task_params[p_key] = p_value
@@ -313,6 +392,9 @@ class Node (object):
             # notification
             if not self.is_workflow_output:
                 task_params['noEmail'] = True
+            # use instant PQs
+            if self.type in ['junction', 'reana']:
+                task_params['runOnInstant'] = True
             # return
             return task_params
         elif self.type == 'phpo':
@@ -352,21 +434,24 @@ class Node (object):
             return {}
         return None
 
-    # get parameters in a loop
-    def get_looping_parameters(self):
+    # get global parameters in the workflow
+    def get_global_parameters(self):
         if self.is_leaf:
             root_inputs = self.upper_root_inputs
         else:
             root_inputs = self.root_inputs
         if root_inputs is None:
-            return None
-        params = {}
+            return None, None
+        loop_params = {}
+        workflow_params = {}
         for k, v in six.iteritems(root_inputs):
             param = k.split('#')[-1]
             m = re.search(r'^param_(.+)', param)
             if m:
-                params[m.group(1)] = v
-        return params
+                loop_params[m.group(1)] = v
+            else:
+                workflow_params[param] = v
+        return loop_params, workflow_params
 
     # get all sub node IDs
     def get_all_sub_node_ids(self, all_ids=None):
@@ -473,7 +558,7 @@ class ConditionItem (object):
 
 
 # convert nodes to workflow
-def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
+def convert_nodes_to_workflow(nodes, workflow_node=None, workflow=None):
     if workflow is None:
         is_top = True
         workflow = Workflow()
@@ -483,7 +568,7 @@ def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
     all_sub_id_work_map = {}
     sub_to_id_map = {}
     cond_dump_str = '  Conditions\n'
-    class_dump_str = '===== Workflow ID:{} ====\n'.format(parent_node.id if parent_node else None)
+    class_dump_str = '===== Workflow ID:{} ====\n'.format(workflow_node.id if workflow_node else None)
     class_dump_str += "  Works\n"
     dump_str_list = []
     # create works or workflows
@@ -491,8 +576,8 @@ def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
         if node.is_leaf:
             # work
             if node.type == 'junction':
-                # FIXME
-                work = ATLASPandaWork(task_parameters=node.task_params)
+                work = ATLASLocalPandaWork(task_parameters=node.task_params)
+                work.add_custom_condition('to_exit', True)
             else:
                 work = ATLASPandaWork(task_parameters=node.task_params)
             workflow.add_work(work)
@@ -512,7 +597,7 @@ def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
             if node.loop:
                 for sub_node in node.sub_nodes:
                     if sub_node.type == 'junction':
-                        cond = Condition(cond=sub_id_work_map[sub_node.id].is_finished)
+                        cond = Condition(cond=sub_id_work_map[sub_node.id].get_not_custom_condition_status)
                         sub_workflow.add_loop_condition(cond)
                         cond_dump_str += '    Loop in ID:{} with terminator ID:{}\n'.format(node.id, sub_node.id)
                         break
@@ -531,13 +616,15 @@ def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
                     else:
                         p_work = all_sub_id_work_map[p_id]
                         str_p_id = sub_to_id_map[p_id]
-                    if len(node.parents) > 1 or isinstance(p_work, Workflow) or node.type == 'junction':
+                    if len(node.parents) > 1 or isinstance(p_work, Workflow) or node.type in ['junction', 'reana']:
                         cond_function = p_work.is_finished
                     else:
                         cond_function = p_work.is_started
-                    cond_func_list.append(cond_function)
-                    cond_dump_str += '    Default Link ID:{} {} -> ID:{}\n'.format(str_p_id, cond_function.__name__,
-                                                                                   node.id)
+                    if cond_function not in cond_func_list:
+                        cond_func_list.append(cond_function)
+                        cond_dump_str += '    Default Link ID:{} {} -> ID:{}\n'.format(str_p_id,
+                                                                                       cond_function.__name__,
+                                                                                       node.id)
                 cond = AndCondition(true_works=[c_work], conditions=cond_func_list)
                 workflow.add_condition(cond)
             else:
@@ -570,7 +657,7 @@ def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
                         base_cond_map[tmp_idx] = cond
                         str_func = "AND ".join(str_func_list)
                         str_cond_map[tmp_idx] = str_func
-                        cond_dump_str += '    Unary Ops  {} ({}) for ID:{}\n'.format(
+                        cond_dump_str += '    Unary Ops {}({}) -> ID:{}\n'.format(
                             cond.__class__.__name__, str_func, node.id)
                         root_condition = cond
                     else:
@@ -618,7 +705,7 @@ def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
                         else:
                             cond = OrCondition(conditions=[l_cond.is_condition_true, r_cond.is_condition_true])
                         base_cond_map[tmp_idx] = cond
-                        cond_dump_str += '    Binary Ops {} ({}) ({}) for ID:{}\n'.format(
+                        cond_dump_str += '    Binary Ops {}({}, {}) for ID:{}\n'.format(
                             cond.__class__.__name__,
                             l_str_func,
                             r_str_func,
@@ -627,8 +714,18 @@ def convert_nodes_to_workflow(nodes, parent_node=None, workflow=None):
                 # set root condition
                 if root_condition:
                     root_condition.true_works = [c_work]
-                    # FIXME
-                    #workflow.add_condition(root_condition)
+                    workflow.add_condition(root_condition)
+    # global parameters
+    if workflow_node:
+        tmp_global, tmp_workflow_global = workflow_node.get_global_parameters()
+        if tmp_global:
+            workflow.set_global_parameters({'user_' + k: tmp_global[k] for k in tmp_global})
+            cond_dump_str += '\n  Looping Globals\n'
+            cond_dump_str += '    {}\n'.format(tmp_global)
+        if tmp_workflow_global:
+            cond_dump_str += '\n  Workflow Globals\n'
+            cond_dump_str += '    {}\n'.format(tmp_workflow_global)
+    # dump strings
     dump_str_list.insert(0, class_dump_str+'\n'+cond_dump_str+'\n\n')
     # return
     if not is_top:
